@@ -1,12 +1,21 @@
 #include "dwarf.h"
-#include "glibc-dl.h"
-#include "CTrace.h"
+#include "dl.h"
 #include <limits.h>     // PATH_MAX
 #include <libdwarf/dwarf.h>
 #include <functional>
-//#include <iostream> // for debug
+#include <memory>
+#include <iostream> // for debug
 
 
+#include <stdio.h>
+
+#ifdef _WIN32
+#include <windows.h>
+#define PANDA_PATH_MAX MAX_PATH
+#else
+#define PANDA_PATH_MAX PATH_MAX
+#endif
+
 namespace panda { namespace backtrace {
 
 static iptr<BacktraceInfo> dl_produce(const RawTrace& buffer);
@@ -16,18 +25,26 @@ static DwarfInfoMap load_dwarf_info(const SharedObjectMap& so_map) {
     DwarfInfoMap result;
     for (auto& so: so_map) {
         auto info = std::make_unique<DwarfInfo>(so);
-        char real_path[PATH_MAX] = {0};
+        string real_path(PANDA_PATH_MAX);
         Dwarf_Error error;
-        auto err = dwarf_init_path(so.name.c_str(), real_path, PATH_MAX, DW_DLC_READ, DW_GROUPNUMBER_ANY,
-                                   nullptr, &info->err_arg, &info->debug, 0, 0, 0, &error);
-        //std::cout << "loading " << so.name << ", code = " << err << "\n";
-        if (err == DW_DLV_OK) {
-            if (info->load()) {
-                ///std::cout << "dwarf info initialized for " << so.name << "\n";
+
+        FILE* file;
+        file = fopen(so.name.c_str(), "rb");
+        DwarfInfo::file_guard_t file_guard (file, [](auto* f){ if (f) {fclose(f); }});
+        int fd = file ? fileno(file) : 0;
+        if (fd > 0) {
+            auto err = dwarf_init_b(fd, DW_DLC_READ, DW_GROUPNUMBER_ANY, nullptr, &info->err_arg, &info->debug, &error);
+            //std::cout << "loading '" << so.name << "', code = " << err << "\n";
+            if (err == DW_DLV_OK) {
+                if (info->load(std::move(file_guard))) {
+                    //std::cout << "dwarf info initialized for " << so.name << "\n";
+                }
+            } else if (err == DW_DLV_ERROR) {
+                //std::cout << "error initializing on " << so.name << " :: " << dwarf_errmsg(error) << "\n";
             }
+            // use DwarfInfoMap independently whether the real file was loaded. So, if file is not found
+            // we still can produce stack frame with address/offset and .so name
         }
-        // use DwarfInfoMap independently whether the real file was loaded. So, if file is not found
-        // we still can produce stack frame with address/offset and .so name
         result.emplace(so.name, std::move(info));
     }
     return result;
@@ -50,7 +67,7 @@ iptr<BacktraceInfo> dl_produce(const RawTrace& buffer) {
             }
         }
     }
-    auto ptr = new CTrace(std::move(frames));
+    auto ptr = new BacktraceInfo(std::move(frames));
     return iptr<BacktraceInfo>(ptr);
 }
 
@@ -70,7 +87,8 @@ DwarfInfo::~DwarfInfo() {
     }
 }
 
-bool DwarfInfo::load() noexcept {
+bool DwarfInfo::load(file_guard_t&& guard_) noexcept {
+    guard = std::move(guard_);
     Dwarf_Error error;
     for(int cu_number = 0;;++cu_number) {
         auto cu = std::make_unique<dwarf::CU>(debug, cu_number);
@@ -99,10 +117,12 @@ bool DwarfInfo::load() noexcept {
 StackframePtr DwarfInfo::resolve(std::uint64_t ip) noexcept {
     if (ip < so_info.begin || ip >= so_info.end) { return StackframePtr(); }
 
-    auto offset = ip - so_info.begin;
-    ///std::cout << "resolving " << std::hex << ip << "/" << offset << " from " <<  so_info.name <<  "\n";
+    auto offset = so_info.get_offset(ip);
+    ///std::cout << "resolving " << std::hex << ip << "/" << offset << " from " <<  so_info.name << ", CUs: " << CUs.size() << "\n";
 
-    for(auto& cu: CUs) {
+    for(auto it = CUs.begin(); it != CUs.end(); ++it){
+        //if (r.is_complete()) std::cout << "hit\n";
+        auto& cu = *it;
         auto r = cu->resolve(offset);
         if (r.is_complete()) { return r.get_frame(ip, so_info); }
     }
@@ -189,7 +209,8 @@ panda::iptr<DieRC> DieRC::discover(Dwarf_Die target) noexcept {
         assert(res == DW_DLV_OK);
         return discover(off, child);
     }
-    assert(0 && "should not happen");
+
+    std::abort();
 }
 
 panda::iptr<DieRC> DieRC::discover(Dwarf_Off target_offset, DieHolder& node) noexcept {
